@@ -47,8 +47,6 @@
 extern "C" {
 #endif
 
-#define JALV_UI_UPDATE_HZ 15
-
 enum PortFlow {
 	FLOW_UNKNOWN,
 	FLOW_INPUT,
@@ -85,16 +83,23 @@ typedef struct {
 } ControlChange;
 
 typedef struct {
-	char* uuid;
-	char* load;
-	bool  dump;
-	bool  generic_ui;
+	char*    uuid;
+	char*    load;
+	uint32_t buffer_size;
+	bool     dump;
+	bool     generic_ui;
 } JalvOptions;
 
 typedef struct {
+	LV2_URID atom_Float;
+	LV2_URID atom_Int;
 	LV2_URID atom_eventTransfer;
+	LV2_URID bufsz_maxBlockLength;
+	LV2_URID bufsz_minBlockLength;
+	LV2_URID bufsz_sequenceSize;
 	LV2_URID log_Trace;
 	LV2_URID midi_MidiEvent;
+	LV2_URID param_sampleRate;
 	LV2_URID time_Position;
 	LV2_URID time_bar;
 	LV2_URID time_barBeat;
@@ -120,6 +125,7 @@ typedef struct {
 	LilvNode* rdfs_label;
 	LilvNode* work_interface;
 	LilvNode* work_schedule;
+	LilvNode* end;  ///< NULL terminator for easy freeing of entire structure
 } JalvNodes;
 
 typedef enum {
@@ -160,16 +166,22 @@ typedef struct {
 	const LilvPlugin*  plugin;         ///< Plugin class (RDF data)
 	LilvUIs*           uis;            ///< All plugin UIs (RDF data)
 	const LilvUI*      ui;             ///< Plugin UI (RDF data)
+	const LilvNode*    ui_type;        ///< Plugin UI type (unwrapped)
 	LilvInstance*      instance;       ///< Plugin instance (shared library)
+	SuilHost*          ui_host;        ///< Plugin UI host support
 	SuilInstance*      ui_instance;    ///< Plugin UI instance (shared library)
 	void*              window;         ///< Window (if applicable)
 	struct Port*       ports;          ///< Port array of size num_ports
+	uint32_t           block_length;   ///< Jack buffer size (block length)
 	size_t             midi_buf_size;  ///< Size of MIDI port buffers
 	uint32_t           num_ports;      ///< Size of the two following arrays:
 	uint32_t           longest_sym;    ///< Longest port symbol
+	uint32_t           ui_update_hz;   ///< Frequency of UI updates
 	jack_nframes_t     sample_rate;    ///< Sample rate
 	jack_nframes_t     event_delta_t;  ///< Frames since last update sent to UI
 	uint32_t           midi_event_id;  ///< MIDI event class ID in event context
+	jack_nframes_t     position;       ///< Transport position in frames
+	bool               rolling;        ///< Transport speed (0=stop, 1=play)
 	bool               buf_size_set;   ///< True iff buffer size callback fired
 	bool               exit;           ///< True iff execution is finished
 	bool               has_ui;         ///< True iff a control UI is present
@@ -184,12 +196,19 @@ jalv_create_ports(Jalv* jalv);
 struct Port*
 jalv_port_by_symbol(Jalv* jalv, const char* sym);
 
-LilvNode*
+const char*
 jalv_native_ui_type(Jalv* jalv);
 
 int
-jalv_open_ui(Jalv*         jalv,
-             SuilInstance* instance);
+jalv_open_ui(Jalv* jalv);
+
+int
+jalv_close_ui(Jalv* jalv);
+
+void
+jalv_ui_instantiate(Jalv*       jalv,
+                    const char* native_ui_type,
+                    void*       parent);
 
 bool
 jalv_ui_is_resizable(Jalv* jalv);
